"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const file_1 = require("./../../utils/file");
const common_1 = require("./../../utils/common");
const isVueFile = (pathname) => {
    const extension = file_1.extractExtension(pathname);
    return extension === 'vue';
};
exports.divideByDirectory = ({ filePathnames, rootDir, }) => {
    const map = new Map();
    filePathnames
        .map(pathname => {
        const relativePathname = pathname.replace(rootDir, '');
        const fileName = file_1.extractFileName(relativePathname);
        const dirName = relativePathname.replace(fileName, '');
        const name = fileName.split('.').shift();
        return {
            absolutePathname: pathname,
            relativePathname: pathname.replace(rootDir, ''),
            dirName,
            fileName,
            name,
        };
    })
        .forEach(context => {
        const { dirName } = context;
        delete context.dirName;
        const pathnamesInDir = map.get(dirName) || [];
        pathnamesInDir.push(context);
        map.set(dirName, pathnamesInDir);
    });
    return map;
};
exports.buildLink = ({ name, relativePathname, distDirPrefix, }) => {
    const slug = common_1.camelToHyphen(name);
    const pathnameList = relativePathname.split('/');
    pathnameList.pop();
    return `/${path_1.default.join(distDirPrefix, ...pathnameList, slug)}/`;
};
exports.default = ({ dirContext, }) => {
    const { rootDir, include, exclude } = dirContext;
    const vueFilePathnames = file_1.getFilePathnamesWithFilter(rootDir, {
        include,
        exclude,
    }).filter(isVueFile);
    const fileContextMap = exports.divideByDirectory({
        filePathnames: vueFilePathnames,
        rootDir,
    });
    const componentContextMap = new Map();
    for (const [dirName, fileContexts] of fileContextMap.entries()) {
        const componentContexts = fileContexts.map(fileContext => {
            const link = exports.buildLink(Object.assign({}, fileContext, { distDirPrefix: dirContext.distDirPrefix }));
            return Object.assign({}, fileContext, { link, catalogPathname: null, existDoc: false });
        });
        componentContextMap.set(dirName, componentContexts);
    }
    return componentContextMap;
};
